#include "kmstcpclient.h"
#include "logencryption.h"
#include "easylogging++.h"

#include <QTcpSocket>
#include <QJsonObject>
#include <QJsonDocument>

KMSTcpClient::KMSTcpClient(QObject *parent): QObject(parent)
{
    m_connected = false;
    m_operateType = 0;
    m_tcpClient = new QTcpSocket();

    connect(m_tcpClient, &QTcpSocket::connected, this, &KMSTcpClient::connected);
    connect(m_tcpClient, &QTcpSocket::disconnected, this, &KMSTcpClient::disconnected);
    connect(m_tcpClient, &QTcpSocket::readyRead, this, [ = ]() {
        emit receData();
    });
    connect(m_tcpClient, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(connectError(QAbstractSocket::SocketError)));
}

KMSTcpClient::~KMSTcpClient()
{
    m_tcpClient->deleteLater();
}

bool KMSTcpClient::connectToServer(QString addr, unsigned int port,  int timeout)
{
    if (this->m_connected) {
        qDebug() << "server connected!";
        return true;
    }

    this->m_tcpClient->connectToHost(addr, port);
    this->m_connected = this->m_tcpClient->waitForConnected(timeout);
    if (!this->m_connected) {
        qDebug() << "kms connect timeout!";
    }
    return this->m_connected;
}

void KMSTcpClient::disconnectServer()
{
    this->m_tcpClient->disconnectFromHost();
    this->m_tcpClient->waitForDisconnected();
}

void KMSTcpClient::sendData(const QJsonObject &obj, const KmsMessageType &messageType)
{
    if (!this->m_connected) {
        qInfo() << "server disconnected!";
        return;
    }
    QVariantMap info = obj.toVariantMap();
    QByteArray data = QJsonDocument(obj).toJson();
    //包头
    KMSHeader header;
    header.len = data.length();
    header.messageType = messageType;
    //组包
    QByteArray array;
    array.append((char *)&header, sizeof(header));
    array.append(data);

    qint64 sendLen = 0;
    qint64 dataLen = array.length();
    while (sendLen < dataLen) {
        sendLen += this->m_tcpClient->write(array.right(dataLen - sendLen));
        if (HeartPkgSend != messageType) {
            qInfo() << "sendData to kms server:" << LogEncryption::encryptionMessage(data);
        }
    }
}

QJsonObject KMSTcpClient::recvData()
{
    QJsonObject jsonObj;
    QByteArray data;
    QJsonParseError parseJsonErr;
    if (!this->m_connected) {
        qInfo() << "server disconnected!";
        return jsonObj;
    }
    //初始化包头
    KMSHeader header;
    memset(&header, 0, sizeof(header));
    //接受包头
    QByteArray array = this->m_tcpClient->read(sizeof(header));
    memcpy(&header, array.data(), sizeof(header));
    qDebug() << "header len" << header.len << endl;
    KMSHeader *kmsHeader = reinterpret_cast<KMSHeader *>(array.data());
    m_operateType = kmsHeader->messageType;
    qDebug() << "m_operateType: " << m_operateType;
    if (VerifyPasswordResult != m_operateType
            && GetActivateInfo != m_operateType
            && WriteLicense != m_operateType) {
        connectError(QAbstractSocket::SocketError::SocketAccessError);
        return jsonObj;
    }

    //循环接受数据
    unsigned int recvlen = 0;
    do {
        data += this->m_tcpClient->read(static_cast<unsigned int>(header.len) - recvlen);
        recvlen = static_cast<unsigned int>(data.length());
    } while (recvlen < static_cast<unsigned int>(header.len));

    QJsonDocument jsonDoc = QJsonDocument::fromJson(data, &parseJsonErr);
    if (!(parseJsonErr.error == QJsonParseError::NoError)) {
        LOG(INFO) << "kms parse json fail！";
        return jsonObj;
    }
    jsonObj = jsonDoc.object();
    return jsonObj;
}

void KMSTcpClient::connected()
{
    this->m_connected = true;
    qDebug() << "server connected!";
}

void KMSTcpClient::disconnected()
{
    this->m_connected = false;
    qDebug() << "server disconnected!";
}

void KMSTcpClient::connectError(QAbstractSocket::SocketError)
{
    this->m_connected = false;
    this->m_tcpClient->abort();
    this->m_tcpClient->close();
    emit kmsServerError();
}

bool KMSTcpClient::isConnected()
{
    return  this->m_connected;
}

qint32 KMSTcpClient::operatorType()
{
    return this->m_operateType;
}
