#include "sysdevice.h"
#include "logencryption.h"
#include "easylogging++.h"

#include <QJsonParseError>
#include <QJsonDocument>
#include <QJsonArray>
#include <QDebug>
#include <QDateTime>
#include <QFile>
#include <QDir>
#include <QDBusInterface>

#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <linux/hdreg.h>

namespace {
static const QString RootPath = "/";
static const QString BootPath = "/boot";
static const QString HostPath = "/host";
static const QString DeviceLoopType = "loop";
static const QString DevicePartType = "part";
static const QString DiskName = "name";
static const QString DiskSerial = "serial";
static const QString DiskType = "type";
static const QString DiskModel = "model";
static const QString DiskMountpoint = "mountpoint";
} // namespace

bool SystemDevice::executeCmd(const QString &cmd, QString &result)
{
    QProcess proc;
    proc.start("bash", {"-c", cmd});
    proc.waitForFinished();
    result = QString::fromLatin1(proc.readAll());
    proc.close();

    return (!result.isEmpty());
}

QString SystemDevice::getMountDiskName()
{
    QString diskMountName;
    QString rootDiskName;
    QString bootDiskName;
    QString hostDiskName;

    if (m_diskDevices.empty())
        getDiskDevices();

    if (m_diskDevices.empty())
        return diskMountName;

    for (auto &dev : m_diskDevices) {
        if (dev.mountpoint == RootPath && (!dev.root.isEmpty()) && DeviceLoopType != dev.type) {
            rootDiskName = dev.root;
            break;
        }
        if (dev.mountpoint == BootPath && (!dev.root.isEmpty()))
            bootDiskName = dev.root;

        if (dev.mountpoint == HostPath && (!dev.root.isEmpty()))
            hostDiskName = dev.root;
    }

    if (!rootDiskName.isEmpty())
        diskMountName = rootDiskName;
    else if (!bootDiskName.isEmpty())
        diskMountName = bootDiskName;
    else if (!hostDiskName.isEmpty())
        diskMountName = hostDiskName;
    else
        qInfo() << "get mount disk name is null!";

    qInfo() << "diskMountName:" << LogEncryption::encryptionMessage(diskMountName);

    return diskMountName;
}

void SystemDevice::getDiskDevices()
{
    QString cmdOut;

    if (false == executeCmd("lsblk -J -bno NAME,SERIAL,TYPE,MODEL,MOUNTPOINT", cmdOut)) {
        LOG(INFO) << "executeCmd return false!";
        return;
    }

    if (cmdOut.isEmpty()) {
        LOG(INFO) << "cmdOut isEmpty!";
        return;
    }

    QJsonParseError err;
    QJsonDocument jsonDoc = QJsonDocument::fromJson(cmdOut.toUtf8(), &err);
    if (err.error != QJsonParseError::NoError) {
        LOG(INFO) << "JSON Parse Error!";
        return;
    }

    if (jsonDoc.isNull()) {
        LOG(INFO) << "jsonDoc isNull!";
        return;
    }

    QJsonObject jsonObj = jsonDoc.object();
    QJsonArray devArray = jsonObj.value("blockdevices").toArray();

    for (int i = 0; i < devArray.size(); i++) {
        DiskDevice dev;
        QJsonObject devObj = devArray.at(i).toObject();
        dev.name = devObj[DiskName].toString();
        dev.serial = devObj[DiskSerial].toString();
        dev.type = devObj[DiskType].toString();
        dev.model = devObj[DiskModel].toString();
        dev.mountpoint = devObj[DiskMountpoint].toString();
        dev.root = dev.name;

        if (dev.mountpoint == RootPath or dev.mountpoint == BootPath) {
            dev.isStartDisk = true;
        }

        m_diskDevices.insert(dev.name, dev);
        parseChildDiskDevice(devObj, dev.root, dev.name);
    }
}

void SystemDevice::parseChildDiskDevice(const QJsonObject &json, const QString &root, const QString &parent)
{
    if (!json["children"].isNull()) {
        QJsonArray devArray = json.value("children").toArray();
        for (int i = 0; i < devArray.size(); i++) {
            DiskDevice dev;
            QJsonObject devObj = devArray.at(i).toObject();
            dev.name = devObj[DiskName].toString();
            dev.serial = devObj[DiskSerial].toString();
            dev.type = devObj[DiskType].toString();
            dev.model = devObj[DiskModel].toString();
            dev.mountpoint = devObj[DiskMountpoint].toString();
            dev.parent = parent;
            dev.root = root;

            if (dev.mountpoint == RootPath or dev.mountpoint == BootPath) {
                dev.isStartDisk = true;
                m_diskDevices[root].isStartDisk = true;
            }
            m_diskDevices.insert(dev.name, dev);
            parseChildDiskDevice(devObj, root, dev.name);
        }
    }
}

QString SystemDevice::diskID(const char *diskname)
{
    QString diskID;
    int fd = 0;
    static struct hd_driveid hd;

    if (getuid() > 0) {
        return diskID;
    }

    if ((fd = open(diskname, O_RDONLY | O_NONBLOCK)) < 0) {
        return diskID;
    }

    if (!ioctl(fd, HDIO_GET_IDENTITY, &hd)) {
        char buffer[1024] = {0};
        int offset = 0;
        size_t modlelen = sizeof(hd.model) / sizeof(hd.model[0]);
        strncpy(buffer + offset, reinterpret_cast<char *>(hd.model), modlelen);
        offset += modlelen;
        size_t seriallen = sizeof(hd.serial_no) / sizeof(hd.serial_no[0]);
        strncpy(buffer + offset, reinterpret_cast<char *>(hd.serial_no), seriallen);
        diskID = QString(buffer);
    }
    close(fd);
    return diskID;
}

QString SystemDevice::getRootDiskPartName()
{
    QString partName;

    if (m_diskDevices.empty())
        getDiskDevices();

    if (m_diskDevices.empty())
        return partName;

    for (auto &dev : m_diskDevices) {
        if (dev.mountpoint == RootPath || dev.mountpoint == BootPath)
            findDiskPartName(dev.mountpoint, dev.name);
    }

    if (m_diskPartName.empty())
        return partName;

    if (m_diskPartName.contains(RootPath)) {
        partName = m_diskPartName.value(RootPath);
    } else if (m_diskPartName.contains(BootPath)) {
        partName = m_diskPartName.value(BootPath);
    } else {
    }

    return partName;
}

void SystemDevice::findDiskPartName(const QString &mountPath, const QString &diskName)
{
    if (!m_diskDevices.contains(diskName))
        return;

    if ((DevicePartType == m_diskDevices.value(diskName).type)
        || (DeviceLoopType == m_diskDevices.value(diskName).type))
        m_diskPartName.insert(mountPath, m_diskDevices.value(diskName).name);

    QString parent = m_diskDevices.value(diskName).parent;

    if (!parent.isEmpty())
        findDiskPartName(mountPath, parent);
}

QString SystemDevice::getSystemBoardID()
{
    QString boardID;
    QString cmdOut;
    QString cmd =
        "dmidecode -t 1 | sed -n '/Product Name:/p;/Serial Number:/p;/UUID:/p' | sed s/[[:space:]]//g|awk  -F ':'  '{print $2}'";
    executeCmd(cmd, cmdOut);

    if (cmdOut.isEmpty())
        return boardID;

    boardID = cmdOut.split("\n").join("#");
    return boardID;
}

QString SystemDevice::getSystemCpuID()
{
    QString cpudevID;
    QString cmdOut;
    executeCmd("dmidecode -t 4 | sed -n '/ID/p;/Version/p' | sed s/[[:space:]]//g | awk -F ':' '{print $2}'", cmdOut);

    if (cmdOut.isEmpty())
        return cpudevID;

    cpudevID = cmdOut.split("\n").join("#");
    return cpudevID;
}

QString SystemDevice::getSystemDiskIDByIoctl()
{
    QString ioctlID;

    if (0 == getuid()) {
        QString diskName = "/dev/" + getMountDiskName();
        ioctlID = diskID(diskName.toStdString().c_str());
    }

    return ioctlID;
}

QString SystemDevice::getSystemDiskIDByLsblk()
{
    QString rootID;
    if (m_diskDevices.empty())
        getDiskDevices();

    if (m_diskDevices.empty())
        return rootID;

    if (isKLU()) {
        rootID = generateDiskInfoByKluCid();
        qInfo() << "generate Disk Info By KluCid";
    } else {
        rootID = generateDiskInfoBylsblk();
        qInfo() << "generate Disk Info By lsblk";
    }

    return rootID;
}

QString SystemDevice::generateDiskInfoByKluCid()
{
    QString rootID;
    QMap<QString, QString> mountDev;
    QString cidInfo = readCIDFile();
    if (cidInfo.isEmpty()) {
        qWarning() << "cid info is empty";
        return rootID;
    }

    for (auto &dev : m_diskDevices) {
        if (dev.mountpoint == RootPath && !dev.root.isEmpty()) {
            DiskDevice mount = m_diskDevices.value(dev.root);
            mountDev[RootPath] = mount.model + "_" + cidInfo;
        } else if (dev.mountpoint == BootPath && !dev.root.isEmpty()) {
            DiskDevice mount = m_diskDevices.value(dev.root);
            mountDev[BootPath] = mount.model + "_" + cidInfo;
        } else {
        }
    }

    if (mountDev.empty()) {
        if (!m_diskDevices.empty())
            rootID = m_diskDevices.begin()->model + "_" + cidInfo;
    } else {
        if (mountDev.contains(RootPath)) {
            rootID = mountDev.value(RootPath);
        } else if (mountDev.contains(BootPath)) {
            rootID = mountDev.value(BootPath);
        } else {
        }
    }
    return rootID;
}

QString SystemDevice::generateDiskInfoBylsblk()
{
    QString rootID;
    QMap<QString, QString> mountDev;
    for (auto &dev : m_diskDevices) {
        if (dev.mountpoint == RootPath && !dev.root.isEmpty()) {
            DiskDevice mount = m_diskDevices.value(dev.root);
            mountDev[RootPath] = mount.model + "_" + mount.serial;
        } else if (dev.mountpoint == BootPath && !dev.root.isEmpty()) {
            DiskDevice mount = m_diskDevices.value(dev.root);
            mountDev[BootPath] = mount.model + "_" + mount.serial;
        } else {
        }
    }

    if (mountDev.empty()) {
        if (!m_diskDevices.empty()) {
            rootID = m_diskDevices.begin()->model + "_" + m_diskDevices.begin()->serial;
        }
    } else {
        if (mountDev.contains(RootPath)) {
            rootID = mountDev.value(RootPath);
        } else if (mountDev.contains(BootPath)) {
            rootID = mountDev.value(BootPath);
        } else {
        }
    }

    return rootID;
}

QString SystemDevice::getSystemInitField(const QString &diskName)
{
    QString tempDiskName = diskName;
    QString field;
    if (tempDiskName.isEmpty())
        tempDiskName = getRootDiskPartName();

    if (tempDiskName.isEmpty())
        return field;

    QString cmd = "dumpe2fs -h /dev/" + tempDiskName + " | grep 'Filesystem created:'";
    executeCmd(cmd, field);

    if (field.isEmpty()) {
        QString lscmd = "ls -lact --time-style '+%Y-%m-%d %H:%M:%S' / | tail -1 |awk '{print $6,$7}'";
        executeCmd(lscmd, field);
        field = "2_" + field;
    } else {
        field = field.split("Filesystem created:").at(1);
        field = "1_" + field;
    }

    if (!field.isEmpty())
        field.replace("\n", "");
    return field;
}

QMap<QString, DiskDevice> SystemDevice::getSystemDiskDevices()
{
    if (m_diskDevices.empty())
        getDiskDevices();

    return m_diskDevices;
}

bool SystemDevice::isDiskRAID()
{
    QString cmdOut;
    executeCmd("lspci | grep RAID", cmdOut);

    if (!cmdOut.isEmpty())
        return true;

    executeCmd("ls -l | grep /dev/md0", cmdOut);

    if (!cmdOut.isEmpty())
        return true;

    return false;
}

bool SystemDevice::isVirtualMachine()
{
    QString cmdOut;
    executeCmd("cat /proc/cpuinfo | grep 'hypervisor'", cmdOut);
    if (!cmdOut.isEmpty())
        return true;

    executeCmd("systemd-detect-virt | xargs", cmdOut);
    if (!cmdOut.isEmpty()) {
        if (!cmdOut.contains("none", Qt::CaseInsensitive))
            return true;
    }

    //华为公有云适配时华为提供2种判断虚拟机方法
    cmdOut = "";
    executeCmd("dmidecode -s system-family | grep -i virtual", cmdOut);
    if (!cmdOut.isEmpty())
        return true;

    executeCmd("dmidecode -s processor-manufacturer | grep -i qemu", cmdOut);
    if (!cmdOut.isEmpty())
        return true;

    return false;
}

bool SystemDevice::isKLU()
{
    QString cmdOut;
    QString cmd = "sudo dmidecode -t 1 | grep 'Product Name'";
    if (!executeCmd(cmd, cmdOut)) {
        qWarning() << "get system product name filed";
        return false;
    }

    // huawei klu
    if (isUFSDiskType() && (cmdOut.contains("L410") || cmdOut.contains("W515")))
        return true;
    return false;
}

QString SystemDevice::readCIDFile()
{
    QFile file(KLUCID_FILE);
    if (!file.open(QIODevice::ReadOnly)) {
        qWarning() << "cid file open failed!";
        return "";
    }
    qInfo() << "cid file open success!";
    QString data = file.readAll().trimmed();
    file.close();

    if (data.isEmpty())
        qWarning() << "cid file empty!";

    qInfo() << "cid info: " << LogEncryption::encryptionMessage(data);
    return data;
}

qint64 SystemDevice::getSystemRunningTime()
{
    QString cmdOut;
    QString cmd = "cat /proc/uptime";
    if (!executeCmd(cmd, cmdOut)) {
        qInfo() << "get systemRunningTime fail";
        return 0;
    }

    qint64 systemRunningTime = cmdOut.remove("\n").split(" ").at(0).split(".").at(0).toLongLong();
    qInfo() << "systemRunningTime: " << LogEncryption::encryptionMessage(QString::number(systemRunningTime));
    return systemRunningTime;
}

qint64 SystemDevice::getLastShutDownTime()
{
    QString cmdOut;
    QString cmd = "last -Fx shutdown|head -1|awk '{print $5,$6,$7,$8,$9}'";
    if (!executeCmd(cmd, cmdOut)) {
        qInfo() << "get lastPoweroffTime fail";
        return 0;
    }

    qint64 lastShutDownTime = QDateTime::fromString(cmdOut.remove("\n")).toTime_t();
    qInfo() << "lastPoweroffTime: " << LogEncryption::encryptionMessage(QString::number(lastShutDownTime));
    return lastShutDownTime;
}

qint64 SystemDevice::getLastRebootTime()
{
    QString cmdOut;
    QString cmd = "last -Fx reboot|head -1|awk '{print $5,$6,$7,$8,$9}'";
    if (!executeCmd(cmd, cmdOut)) {
        qInfo() << "get lastRebootTime fail";
        return 0;
    }

    qint64 lastRebootTime = QDateTime::fromString(cmdOut.remove("\n")).toTime_t();
    qInfo() << "lastRebootTime: " << LogEncryption::encryptionMessage(QString::number(lastRebootTime));
    return lastRebootTime;
}

// 5.1.9
QString SystemDevice::stripValue(QString value)
{
    if (value.length() >= 2) {
        if (value.at(value.length() - 1) == '"') {
            value = value.left(value.length() - 1);
        }

        if (value.at(0) == '"') {
            value = value.right(value.length() - 1);
        }
    }
    return value;
}

QList<DiskInfo> SystemDevice::previousSp1ParseLsblk(const QString &lsblkOut)
{
    QList<DiskInfo> disks;
    auto entrys = lsblkOut.split("\n");
    for (auto entry : entrys) {
        DiskInfo info;
        auto propertys = entry.split(" ");
        for (auto pro : propertys) {
            auto kv = pro.split("=");
            if (kv.length() != 2) {
                continue;
            }
            info.insert(kv.value(0), stripValue(kv.value(1)));
        }
        disks << info;
    }
    return disks;
}

QList<DiskInfo> SystemDevice::getPreviousSp1MountDisk()
{
    QProcess lsblk;
    lsblk.setProgram("lsblk");
    lsblk.setArguments(QStringList() << "-P");
    lsblk.start();
    lsblk.waitForFinished();

    auto rootMountpoint = QStringList() << RootPath << BootPath;
    auto lsblkOut = QString::fromLatin1(lsblk.readAll());
    auto disks = previousSp1ParseLsblk(lsblkOut);
    QList<DiskInfo> mountDisk;
    for (auto disk : disks) {
        auto mp = disk.value("MOUNTPOINT");

        if (!rootMountpoint.contains(mp)) {
            continue;
        }
        mountDisk << disk;
    }
    return mountDisk;
}

QList<DiskInfo> SystemDevice::getPhyDisk()
{
    QProcess lsblk;
    lsblk.setProgram("lsblk");
    lsblk.setArguments(QStringList() << "-P"
                                     << "-d");
    lsblk.start();
    lsblk.waitForFinished();

    auto lsblkOut = QString::fromLatin1(lsblk.readAll());
    return previousSp1ParseLsblk(lsblkOut);
}

QString getDiskMaj(const DiskInfo &disk)
{
    auto mm = disk.value("MAJ:MIN");
    auto mms = mm.split(":");
    if (mms.length() != 2) {
        return "";
    }
    return mms.first();
}

QString SystemDevice::getPreviousSp1RootDisk()
{
    auto mountDisk = getPreviousSp1MountDisk();
    auto phyDisk = getPhyDisk();

    for (auto disk : mountDisk) {
        auto maj = getDiskMaj(disk);
        for (auto phy : phyDisk) {
            auto phyMaj = getDiskMaj(phy);
            if (phyMaj == maj) {
                return QString("/dev/%1").arg(phy.value("NAME"));
            }
        }
    }
    return "";
}

QString SystemDevice::fillMachineIDAsMachineCode()
{
    QString machineID;
    QFile machineIDFile("/etc/machine-id");
    if (false == machineIDFile.open(QIODevice::ReadOnly)) {
        machineIDFile.close();
        return machineID;
    }
    machineID = machineIDFile.readAll().left(MD5_DIGEST_LENGTH);
    machineIDFile.close();
    qDebug() << "machine-id" << machineID;
    return machineID;
}

QString SystemDevice::getPreviousSp1DiskInfo(bool &isGetRealDiskInfo)
{
    QString diskInfo;
    QString rootDiskName = getPreviousSp1RootDisk();
    qDebug() << "5.1.9-version diskName : " << rootDiskName;
    diskInfo = diskID(rootDiskName.toStdString().c_str());
    if (diskInfo.isEmpty()) {
        qDebug() << "try to read machine code";
        diskInfo = fillMachineIDAsMachineCode();
        isGetRealDiskInfo = false;
    }
    return diskInfo;
}

// sp1
QString SystemDevice::fillSp1MachineIDAsMachineCode()
{
    QString machineID;
    QFile machineIDFile("/etc/machine-id");
    if (false == machineIDFile.open(QIODevice::ReadOnly)) {
        machineIDFile.close();
        return machineID;
    }
    machineID = machineIDFile.readAll();
    machineIDFile.close();
    qDebug() << "machine-id" << machineID;
    return machineID;
}

QString SystemDevice::getSp1DiskInfo(bool &isGetRealDiskInfo)
{
    QString diskInfo;
    QString rootDiskName = getPreviousSp1RootDisk();
    qDebug() << "sp1 diskName : " << rootDiskName;
    diskInfo = diskID(rootDiskName.toStdString().c_str());
    if (diskInfo.isEmpty()) {
        qDebug() << "try to read machine code";
        diskInfo = fillSp1MachineIDAsMachineCode();
        isGetRealDiskInfo = false;
    }
    return diskInfo;
}

// sp1-update1
bool SystemDevice::isVirtualMachineSp1Update1()
{
    QFile cpuInfoFile("/proc/cpuinfo");
    if (cpuInfoFile.open(QIODevice::ReadOnly)) {
        const QString conntent = cpuInfoFile.readAll();
        return conntent.contains("hypervisor");
    } else {
        return false;
    }
}

void SystemDevice::resetMachineId()
{
    system("rm -f /etc/machine-id");
    system("dbus-uuidgen --ensure=/etc/machine-id");
    system("rm -f /var/lib/dbus/machine-id");
    system("dbus-uuidgen --ensure");

    //判断/var/nfs路径是否存在
    QDir dir = QDir("/var/nfs");
    if (!dir.exists()) {
        dir.mkpath("/var/nfs");
    }

    system("cp /etc/machine-id /var/nfs/.machine-id ");
}

QString SystemDevice::getSp1Update1MountDisk()
{
    QProcess lsblk;
    lsblk.setProgram("lsblk");
    lsblk.setArguments(QStringList() << "-o"
                                     << "PKNAME,TYPE,MOUNTPOINT"
                                     << "-P");
    lsblk.start();
    lsblk.waitForFinished();
    auto lsblkOut = QString::fromLatin1(lsblk.readAll());
    lsblk.close();

    QList<DiskInfo> mountDisk = parseLsblk(lsblkOut);

    QFileInfo dir("/run/live/");
    if(dir.isDir())
    {
        QProcess mount_path;
        mount_path.start("cat /tmp/rootMountPoint");
        mount_path.waitForFinished();
        QString mountpoint_path = QString::fromLatin1(mount_path.readAllStandardOutput());
        mount_path.close();
        LOG(INFO) << "mountpoint_path:" << mountpoint_path;

        for (auto disk : mountDisk)
        {
            QString mountPoint = disk.value("MOUNTPOINT");
            if(mountPoint == mountpoint_path)
            {
                QString mountDiskName = disk.value("PKNAME");
                LOG(INFO) << "mountDiskName: " << mountDiskName;
                return mountDiskName;
            }
        }
    }
    else
    {
        for (auto disk : mountDisk) {
            QString mountPoint = disk.value("MOUNTPOINT");
            if ((RootPath == mountPoint) && 0 == disk.value("TYPE").compare(DevicePartType, Qt::CaseInsensitive)) {
                qDebug() << "MountpointInfo:" << disk;
                QString mountDiskName = disk.value("PKNAME");
                return mountDiskName;
            }
        }

        for (auto disk : mountDisk) {
            QString mountPoint = disk.value("MOUNTPOINT");
            if ((BootPath == mountPoint) || (HostPath == mountPoint)) {
                qDebug() << "MountpointInfo:" << disk;
                QString mountDiskName = disk.value("PKNAME");
                return mountDiskName;
            }
        }

        for (auto disk : mountDisk) {
            if (disk.value("TYPE").compare("disk", Qt::CaseInsensitive)) {
                qDebug() << "First diskInfo:" << disk;
                QString mountDiskName = disk.value("NAME");
                return mountDiskName;
            }
        }

        qDebug() << "Can't find mountDiskName:" << mountDisk;
    }

    return "";
}

bool SystemDevice::isUFSDiskType()
{
    QString tempDiskName = getMountDiskName();
    QString field;
    QString cmd = "udevadm info /dev/" + tempDiskName + " | grep -E 'ID_PATH_TAG=|ID_PATH='";
    executeCmd(cmd, field);
    if (field.isEmpty() || !field.contains("ufs", Qt::CaseInsensitive))
        return false;

    return true;
}

QString SystemDevice::getDiskIdByDBus(QString &diskName)
{
    QString path = QString("%1%2").arg("/org/freedesktop/UDisks2/block_devices/").arg(diskName);
    QDBusInterface udisk("org.freedesktop.UDisks2", path, "org.freedesktop.UDisks2.Block",
                         QDBusConnection::systemBus());

    QVariant currentPath = udisk.property("Drive");
    QDBusInterface currentDisk("org.freedesktop.UDisks2", currentPath.value<QDBusObjectPath>().path(),
                               "org.freedesktop.UDisks2.Drive", QDBusConnection::systemBus());

    QString strDrive = currentDisk.property("Id").toString();

    if (strDrive.isEmpty()) {
        return "";
    }

    strDrive = strDrive.replace("_", "");
    strDrive = strDrive.replace("-", "");
    strDrive = strDrive.replace(" ", "");
    strDrive = strDrive.replace(".", "");
    qDebug() << "Model&&SerialNo by dbus:" << strDrive;
    return strDrive;
}

QList<DiskInfo> SystemDevice::parseLsblk(const QString &lsblkOut)
{
    QList<DiskInfo> disks;
    auto entrys = lsblkOut.split("\n", QString::SkipEmptyParts);
    for (auto entry : entrys) {
        DiskInfo info;
        auto propertys = entry.split(" ", QString::SkipEmptyParts);
        for (auto pro : propertys) {
            auto kv = pro.split("=", QString::SkipEmptyParts);
            if (kv.length() != 2) {
                continue;
            }
            info.insert(kv.value(0), stripValue(kv.value(1)));
        }
        disks << info;
    }
    return disks;
}

QString SystemDevice::getDiskIdByLsblk(QString &parentName)
{
    QProcess lsblk;
    lsblk.setProgram("lsblk");
    lsblk.setArguments(QStringList() << "-o"
                                     << "NAME,MODEL,SERIAL"
                                     << "-P");
    lsblk.start();
    lsblk.waitForFinished();
    auto lsblkOut = QString::fromLatin1(lsblk.readAll());
    lsblk.close();

    QList<DiskInfo> diskInfo = parseLsblk(lsblkOut);

    for (auto disk : diskInfo) {
        QString diskName = disk.value("NAME");
        if (parentName == diskName) {
            QString strDrive = disk.value("MODEL") + disk.value("SERIAL");
            strDrive = strDrive.replace("_", "");
            strDrive = strDrive.replace("-", "");
            strDrive = strDrive.replace(" ", "");
            strDrive = strDrive.replace(".", "");
            qDebug() << "Model&&SerialNo by lsblk:" << strDrive;
            return strDrive;
        }
    }
    qDebug() << "Can't get diskinfo by lsblk:" << diskInfo;
    return "";
}

QString SystemDevice::getSp1Update1DiskInfo(bool &isGetRealDiskInfo)
{
    QString diskInfo;
    // is Virtual Machine
    if (isVirtualMachineSp1Update1()) {
        isGetRealDiskInfo = false;
        QFile machineidVarFile("/var/nfs/.machine-id");
        QFile machineidFile("/etc/machine-id");
        if (!machineidVarFile.open(QIODevice::ReadOnly) || !machineidFile.open(QIODevice::ReadOnly)) {
            resetMachineId();
        } else {
            const QString conntent = machineidVarFile.readAll();
            QString strMachineid = machineidFile.readAll();
            if (0 != strMachineid.compare(conntent)) {
                resetMachineId();
            }
        }
        diskInfo = fillSp1MachineIDAsMachineCode();
        if (diskInfo.isEmpty()) {
            qDebug() << "Fail:Virtual can't open machineIDFile";
            return "";
        }
    } else {
        if (0 == getuid()) {
            // use ioctl
            QString mountDiskName = getSp1Update1MountDisk();
            QString diskName = "/dev/" + mountDiskName;
            diskInfo = diskID(diskName.toStdString().c_str());

            if (diskInfo.isEmpty()) {
                diskInfo = getDiskIdByDBus(mountDiskName);
            }

            if (diskInfo.isEmpty()) {
                diskInfo = getDiskIdByLsblk(mountDiskName);
            }
        } else {
            qDebug() << "program no root permission.";
        }

        if (diskInfo.isEmpty()) {
            isGetRealDiskInfo = false;
            qDebug() << "try to read /etc/machine-id file.";
            if (fillSp1MachineIDAsMachineCode().isEmpty()) {
                qDebug() << "Fail:Phymachine can't open machineIDFile";
                return "";
            }
        } else {
            qDebug() << "read hard-drive info success.";
        }
    }
    return diskInfo;
}
