#include "systeminfo.h"
#include "logencryption.h"
#include "easylogging++.h"

#include <QDebug>
#include <QSettings>
#include <QProcess>
#include <QFile>
#include <QDir>
#include <QDBusInterface>
#include <QCryptographicHash>
#include <QRegularExpression>
#include <QDateTime>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>

#include <openssl/md5.h>
#include <openssl/sha.h>

#include <linux/hdreg.h>
#include <sys/ioctl.h>

SystemInfo::SystemInfo()
{
    m_systemDevice = new SystemDevice();
    initHardwareInfo();
    m_rootDiskPartName = m_systemDevice->getRootDiskPartName();
}

SystemInfo::~SystemInfo()
{
    if (m_systemDevice != nullptr) {
        delete m_systemDevice;
        m_systemDevice = nullptr;
    }
}

void SystemInfo::initHardwareInfo()
{
    m_hardWareInfo.a1 = getDiskCodeByIoctl();
    m_hardWareInfo.a2 = getDiskCodeByIoctlOrDBusOrLsblk();
    m_hardWareInfo.a3 = getDiskCodeByIoctlOrDBus();
    m_hardWareInfo.a4 = getDiskInfoByAllDBus();
    m_hardWareInfo.a5 = getDiskInfoByLsblk();
    m_hardWareInfo.a6 = getDiskCodeByDbusAndLsblk();
    m_hardWareInfo.systemId = getInitSystemID();
    m_hardWareInfo.board_cpu = getSystemBoardAndCpuCode();
    m_hardWareInfo.mac = getMacAddress();
    m_hardWareInfo.is_vm = getIsVirtualMachine();
    m_hardWareInfo.is_raid = getIsRAID();
    m_hardWareInfo.previousSp1MachineCode = getPreviousSp1MachineCode();
    m_hardWareInfo.sp1MachineCode = getSp1MachineCode();
    m_hardWareInfo.sp1Update1MachineCode = getSp1Update1MachineCode();
}

QString SystemInfo::getRootDiskPartName()
{
    return m_rootDiskPartName;
}

QString SystemInfo::stripValue(const QString &value)
{
    QString noQuotes = value;
    if (noQuotes.length() >= 2) {
        if (noQuotes.at(noQuotes.length() - 1) == '"') {
            noQuotes = noQuotes.left(noQuotes.length() - 1);
        }

        if (noQuotes.at(0) == '"') {
            noQuotes = noQuotes.right(noQuotes.length() - 1);
        }
    }
    return noQuotes;
}

QList<DiskInfo> SystemInfo::parseLsblk(const QString &lsblkOut)
{
    QList<DiskInfo> disks;
    auto entrys = lsblkOut.split("\n", QString::SkipEmptyParts);
    for (auto entry : entrys) {
        DiskInfo info;
        auto propertys = entry.split(" ", QString::SkipEmptyParts);
        for (auto pro : propertys) {
            auto kv = pro.split("=", QString::SkipEmptyParts);
            if (kv.length() != 2) {
                continue;
            }
            info.insert(kv.value(0), stripValue(kv.value(1)));
        }
        disks << info;
    }
    return disks;
}

QString SystemInfo::getMountDisk()
{
    QProcess lsblk;
    lsblk.setProgram("lsblk");
    lsblk.setArguments(QStringList() << "-o"
                                     << "PKNAME,NAME,TYPE,MOUNTPOINT"
                                     << "-P");
    lsblk.start();
    lsblk.waitForFinished();
    auto lsblkOut = QString::fromLatin1(lsblk.readAll());
    lsblk.close();

    QList<DiskInfo> mountDisk = parseLsblk(lsblkOut);

    QFileInfo dir("/run/live/");
    if(dir.isDir())
    {
        QProcess mount_path;
        mount_path.start("cat /tmp/rootMountPoint");
        mount_path.waitForFinished();
        QString mountpoint_path = QString::fromLatin1(mount_path.readAllStandardOutput());
        mount_path.close();
        LOG(INFO) << "mountpoint_path:" << mountpoint_path;

        for (auto disk : mountDisk)
        {
            QString mountPoint = disk.value("MOUNTPOINT");
            if(mountPoint == mountpoint_path)
            {
                QString mountDiskName = disk.value("PKNAME");
                LOG(INFO) << "mountDiskName: " << mountDiskName;
                return mountDiskName;
            }
        }

    }
    else
    {
        for (auto disk : mountDisk) {
            QString mountPoint = disk.value("MOUNTPOINT");
            if (("/" == mountPoint) && 0 == disk.value("TYPE").compare("part", Qt::CaseInsensitive)) {
                qDebug() << "MountpointInfo:" << disk;
                QString mountDiskName = disk.value("PKNAME");
                return mountDiskName;
            }
        }

        for (auto disk : mountDisk) {
            QString mountPoint = disk.value("MOUNTPOINT");
            if (("/boot" == mountPoint) || ("/host" == mountPoint)) {
                qDebug() << "MountpointInfo:" << disk;
                QString mountDiskName = disk.value("PKNAME");
                return mountDiskName;
            }
        }

        for (auto disk : mountDisk) {
            if (disk.value("TYPE").compare("disk", Qt::CaseInsensitive)) {
                qDebug() << "First diskInfo:" << disk;
                QString mountDiskName = disk.value("NAME");
                return mountDiskName;
            }
        }

        qInfo() << "Can't find mountDiskName:" << mountDisk;
    }

    return "";
}

void SystemInfo::idBufferMd5Update(unsigned char *id, const char *buffer)
{
    MD5_CTX c;
    unsigned char out[MD5_DIGEST_LENGTH] = {0};
    MD5_Init(&c);
    MD5_Update(&c, buffer, 1024);
    MD5_Final(out, &c);
    memcpy(id, out, MD5_DIGEST_LENGTH);
}

QString SystemInfo::encodeDiskID(unsigned char *id)
{
    for (int i = 0; i < MD5_DIGEST_LENGTH / 2; i++) {
        id[i] = id[i] / 2 + id[MD5_DIGEST_LENGTH - 1 - i] / 2;
    }

    QString machine_code;
    for (int i = 0; i < 4; i++) {
        machine_code.append(QString().sprintf("%02x", id[i * 4]));
        machine_code.append(QString().sprintf("%02x", id[i * 4 + 1]));
        if (i < 3) {
            machine_code += "-";
        }
    }
    qInfo() << "machineID:" << machine_code;
    return machine_code;
}

HardWareInfo SystemInfo::getHardWareInfo() const
{
    return m_hardWareInfo;
}

QString SystemInfo::getUntreatedDiskInfoByDBus()
{
    QString mountDiskName = getMountDisk();
    QString path = QString("%1%2").arg("/org/freedesktop/UDisks2/block_devices/").arg(mountDiskName);
    QDBusInterface udisk("org.freedesktop.UDisks2", path, "org.freedesktop.UDisks2.Block",
                         QDBusConnection::systemBus());
    QByteArray dbusByIdModelAndSerial;
    // HUAWEI KLU
    if (m_systemDevice->isKLU()) {
        QVariant currentPath = udisk.property("Drive");
        QDBusInterface currentDisk("org.freedesktop.UDisks2", currentPath.value<QDBusObjectPath>().path(),
                                   "org.freedesktop.UDisks2.Drive", QDBusConnection::systemBus());
        QByteArray cidInfo = m_systemDevice->readCIDFile().toLatin1();
        dbusByIdModelAndSerial = currentDisk.property("Model").toByteArray();
        if (cidInfo.isEmpty() || dbusByIdModelAndSerial.isEmpty())
            return "";
        dbusByIdModelAndSerial = dbusByIdModelAndSerial + cidInfo;
    } else {
        dbusByIdModelAndSerial = udisk.property("Id").toByteArray();
    }

    return dbusByIdModelAndSerial;
}

QString SystemInfo::getTreatedDiskInfoByDBus()
{
    QString mountDiskName = getMountDisk();
    QString path = QString("%1%2").arg("/org/freedesktop/UDisks2/block_devices/").arg(mountDiskName);
    QDBusInterface udisk("org.freedesktop.UDisks2", path, "org.freedesktop.UDisks2.Block",
                         QDBusConnection::systemBus());
    QVariant currentPath = udisk.property("Drive");
    QDBusInterface currentDisk("org.freedesktop.UDisks2", currentPath.value<QDBusObjectPath>().path(),
                               "org.freedesktop.UDisks2.Drive", QDBusConnection::systemBus());

    // huaweiKLU
    QString dbusNoByIdModelAndSerial;
    if (m_systemDevice->isKLU()) {
        QString cidInfo = m_systemDevice->readCIDFile();
        dbusNoByIdModelAndSerial = currentDisk.property("Model").toString();
        if (cidInfo.isEmpty() || dbusNoByIdModelAndSerial.isEmpty())
            return "";
        dbusNoByIdModelAndSerial = dbusNoByIdModelAndSerial + cidInfo;
    } else {
        dbusNoByIdModelAndSerial = currentDisk.property("Id").toString();
        if (dbusNoByIdModelAndSerial.isEmpty())
            return "";
    }

    dbusNoByIdModelAndSerial.remove(QRegularExpression(QString("[\\-_ \\.]")));
    return dbusNoByIdModelAndSerial;
}

QString SystemInfo::getDiskInfoByLsblk()
{
    QString lsblkModelAndSerial = m_systemDevice->getSystemDiskIDByLsblk();
    if (lsblkModelAndSerial.isEmpty())
        return "";
    lsblkModelAndSerial.remove(QRegularExpression(QString("[\\-_ \\.]")));
    return lsblkModelAndSerial;
}

QString SystemInfo::createDiskCodeByDiskInfo(const QString &diskInfo)
{
    //以固定1024长度md5加密磁盘信息，超过1024将被截取，只在此处适用
    MD5_CTX c;
    MD5_Init(&c);
    MD5_Update(&c, diskInfo.leftJustified(1024, '\0', true).toStdString().data(), 1024);
    unsigned char id[MD5_DIGEST_LENGTH] = {0};
    MD5_Final(id, &c);
    return encodeDiskID(id);
}

QString SystemInfo::createMd5String(const QString &source)
{
    QString sourceHash = QCryptographicHash::hash(source.toUtf8(), QCryptographicHash::Md5).toHex();
    return sourceHash;
}

QString SystemInfo::getDiskCodeByDbusAndLsblk()
{
    QString strDiskIdByLsblk = getDiskInfoByLsblk();

    if (!strDiskIdByLsblk.isEmpty())
        return createMd5String(strDiskIdByLsblk);

    QString strDiskIdByDBus = getTreatedDiskInfoByDBus();

    return createMd5String(strDiskIdByDBus);
}

//通过ioctl函数获取硬盘信息
QString SystemInfo::getDiskCodeByIoctl()
{
    QString diskInfo = m_systemDevice->getSystemDiskIDByIoctl();
    qInfo() << "ioctl:" << LogEncryption::encryptionMessage(diskInfo);

    return createDiskCodeByDiskInfo(diskInfo);
}

// A2
QString SystemInfo::getDiskCodeByIoctlOrDBusOrLsblk()
{
    QString diskInfo = m_systemDevice->getSystemDiskIDByIoctl();

    if (diskInfo.isEmpty())
        diskInfo = getTreatedDiskInfoByDBus();

    if (diskInfo.isEmpty())
        diskInfo = getDiskInfoByLsblk();

    return createDiskCodeByDiskInfo(diskInfo);
}

// A3
QString SystemInfo::getDiskCodeByIoctlOrDBus()
{
    QString diskInfo = m_systemDevice->getSystemDiskIDByIoctl();

    if (diskInfo.isEmpty())
        diskInfo = getUntreatedDiskInfoByDBus();

    return createDiskCodeByDiskInfo(diskInfo);
}

QString SystemInfo::getDiskInfoByAllDBus()
{
    QString diskInfo = getUntreatedDiskInfoByDBus();
    diskInfo += "," + getTreatedDiskInfoByDBus();

    if (m_systemDevice->isKLU() && !diskInfo.isEmpty())
        diskInfo = "HW_KLU," + diskInfo;

    qInfo() << "get diskInfo by dbus: " << LogEncryption::encryptionMessage(diskInfo);
    return diskInfo;
}

QString SystemInfo::getInitSystemID(const QString &mountPoint)
{
    QString strSystemID;
    QString systemTime = m_systemDevice->getSystemInitField(mountPoint).trimmed();

    QStringList initTimeFields = systemTime.split('_');
    if (initTimeFields.size() > 1) {
        QDateTime systemDateTime;
        if (initTimeFields.at(0) == "1")
            systemDateTime = QDateTime::fromString(initTimeFields.at(1));
        else
            systemDateTime = QDateTime::fromString(initTimeFields.at(1), "yyyy-MM-dd hh:mm:ss");

        uint systemID = QDateTime::fromMSecsSinceEpoch(systemDateTime.toMSecsSinceEpoch(), Qt::UTC).toTime_t();
        strSystemID = QString::number(systemID);
    }

    QString systemBoardID = m_systemDevice->getSystemBoardID();
    QString systemCpuID = m_systemDevice->getSystemCpuID();

    strSystemID += systemBoardID + systemCpuID;

    QString strDiskIdByLsblk = getDiskInfoByLsblk();

    if (!strDiskIdByLsblk.isEmpty()) {
        strSystemID += strDiskIdByLsblk;
    } else {
        strSystemID += getTreatedDiskInfoByDBus();
    }

    qInfo() << "systemTime: " << LogEncryption::encryptionMessage(systemTime);
    qInfo() << "systemBoardID: " << LogEncryption::encryptionMessage(systemBoardID);
    qInfo() << "strDiskIdByLsblk" << LogEncryption::encryptionMessage(strDiskIdByLsblk);
    qInfo() << "systemCpuID" << LogEncryption::encryptionMessage(systemCpuID);
    return createMd5String(strSystemID);
}

QString SystemInfo::getMacAddress(bool deleteColon)
{
    //获得所有的虚拟网卡
    QStringList allVirtualMac;
    QProcess *process = new QProcess(this);

    connect(process,
            static_cast<void (QProcess::*)(int exitCode, QProcess::ExitStatus exitStatus)>(&QProcess::finished), this,
            [&](int exitCode, QProcess::ExitStatus exitStatus) {
                if (exitCode == 0 && exitStatus == QProcess::ExitStatus::NormalExit) {
                    allVirtualMac << QString::fromLocal8Bit(process->readAllStandardOutput())
                                         .split(QRegExp("[\r\n]"), QString::SkipEmptyParts);
                    qInfo() << "all virtual mac " << LogEncryption::encryptionMessage(allVirtualMac);
                }
                process->deleteLater();
            });

    process->start("ls", QStringList() << "/sys/devices/virtual/net/");
    //等待process执行结束
    process->waitForFinished();

    QList<QNetworkInterface> nets = QNetworkInterface::allInterfaces(); // 获取所有网络接口列表
    QStringList physicsMacDevices;
    QStringList virtualMacDevices;
    QString tempMacAddr;

    for (int i = 0; i < nets.count(); i++) {
        // 非回环地址:127.0.0.1
        if (!nets[i].flags().testFlag(QNetworkInterface::IsLoopBack)) {
            tempMacAddr = nets[i].hardwareAddress();
            if (deleteColon)
                tempMacAddr.remove(":");
            qInfo() << "macCount--strMacAddr" << i << LogEncryption::encryptionMessage(tempMacAddr);
            if (allVirtualMac.contains(nets[i].humanReadableName())) {
                virtualMacDevices.append(tempMacAddr);
                qInfo() << "virtual address" << LogEncryption::encryptionMessage(tempMacAddr);
            } else {
                physicsMacDevices.append(tempMacAddr);
                qInfo() << "Physics address" << LogEncryption::encryptionMessage(tempMacAddr);
            }
        }

        if (physicsMacDevices.size() == 2)
            break;
    }

    QString macAddr;
    if (!physicsMacDevices.isEmpty()) {
        macAddr = physicsMacDevices[0]; // QString(physicsMacDevices.join(","));
        qInfo() << "return physics macAddr" << LogEncryption::encryptionMessage(macAddr);
        return macAddr;
    }

    if (!virtualMacDevices.isEmpty()) {
        if (virtualMacDevices.size() > 1)
            macAddr = virtualMacDevices.at(0); // + "," + virtualMacDevices.at(1);
        else
            macAddr = virtualMacDevices.at(0);
    }
    qInfo() << "return macAddr" << LogEncryption::encryptionMessage(macAddr);
    return macAddr;
}

QString SystemInfo::getSystemBoardAndCpuCode()
{
    QString boardCpu = m_systemDevice->getSystemBoardID();
    boardCpu += m_systemDevice->getSystemCpuID();
    boardCpu = createMd5String(boardCpu);
    return boardCpu;
}

int SystemInfo::getIsRAID()
{
    if (m_systemDevice->isDiskRAID())
        return 1;
    return 2;
}

int SystemInfo::getIsVirtualMachine()
{
    if (m_systemDevice->isVirtualMachine())
        return 1;
    return 2;
}

bool SystemInfo::getIsKLU()
{
    return m_systemDevice->isKLU();
}

QString SystemInfo::getKLUCID()
{
    return m_systemDevice->readCIDFile();
}

QMap<QString, QVariant> SystemInfo::getSystemDeviceInfoMap()
{
    QMap<QString, QVariant> sysDevicesMap;
    QMap<QString, DiskDevice> diskDevices = m_systemDevice->getSystemDiskDevices();

    QMap<QString, QVariant> deviceMap;
    for (auto disk : diskDevices) {
        if (disk.type != "disk") {
            continue;
        }

        QMap<QString, QVariant> diskMap;
        diskMap.insert("a1", getDiskCodeByIoctl());
        diskMap.insert("a2", getDiskCodeByIoctlOrDBusOrLsblk());
        diskMap.insert("a3", getDiskCodeByIoctlOrDBus());
        diskMap.insert("a4", getDiskInfoByAllDBus());
        diskMap.insert("a5", getDiskInfoByLsblk());
        diskMap.insert("a6", getDiskCodeByDbusAndLsblk());
        diskMap.insert("is_system", 0);
        if (disk.isStartDisk) {
            diskMap.insert("is_system", 1);
        }
        deviceMap.insert(disk.name, diskMap);
    }
    sysDevicesMap.insert("disk", deviceMap);
    sysDevicesMap.insert("board_cpu", getSystemBoardAndCpuCode());
    sysDevicesMap.insert("mac", getMacAddress(false));
    sysDevicesMap.insert("is_vm", getIsVirtualMachine());
    sysDevicesMap.insert("is_raid", getIsRAID());

    return sysDevicesMap;
}

// 5.1.9(1000) machineCode
QString SystemInfo::createPreviousSp1DiskInfo(const QString &diskInfo)
{
    unsigned char id[MD5_DIGEST_LENGTH] = {0};
    memcpy(id, diskInfo.toLatin1().data(), MD5_DIGEST_LENGTH);
    return encodeDiskID(id);
}

QString SystemInfo::getPreviousSp1MachineCode()
{
    bool isGetRealDiskInfo = true;
    QString diskInfo = m_systemDevice->getPreviousSp1DiskInfo(isGetRealDiskInfo);
    qDebug() << "5.1.9 diskinfo : " << diskInfo;
    if (isGetRealDiskInfo) {
        return createDiskCodeByDiskInfo(diskInfo);
    }
    return createPreviousSp1DiskInfo(diskInfo);
}

// sp1(1010) machineCode
QString SystemInfo::createSp1DiskInfo(const QString &diskInfo)
{
    MD5_CTX c;
    MD5_Init(&c);
    MD5_Update(&c, diskInfo.leftJustified(1024, '\0', true).toStdString().data(), 1024);
    unsigned char id[MD5_DIGEST_LENGTH] = {0};
    MD5_Final(id, &c);
    return encodeDiskID(id);
}

QString SystemInfo::getSp1MachineCode()
{
    bool isGetRealDiskInfo = true;
    QString diskInfo = m_systemDevice->getSp1DiskInfo(isGetRealDiskInfo);
    qDebug() << "sp1 diskinfo : " << diskInfo;
    if (isGetRealDiskInfo) {
        return createDiskCodeByDiskInfo(diskInfo);
    }
    return createSp1DiskInfo(diskInfo);
}

// sp1-update1(1011) machineCode
QString SystemInfo::getSp1Update1MachineCode()
{
    bool isGetRealDiskInfo = true;
    QString diskInfo = m_systemDevice->getSp1Update1DiskInfo(isGetRealDiskInfo);
    qDebug() << "sp1-update1 diskinfo : " << diskInfo;
    if (isGetRealDiskInfo) {
        return createDiskCodeByDiskInfo(diskInfo);
    }
    return createSp1DiskInfo(diskInfo);
}
